package com.luojigou.xiaodou

import android.annotation.SuppressLint
import android.app.Service
import android.bluetooth.BluetoothAdapter
import android.bluetooth.BluetoothDevice
import android.bluetooth.BluetoothGatt
import android.bluetooth.BluetoothGattCallback
import android.bluetooth.BluetoothGattCharacteristic
import android.bluetooth.BluetoothManager
import android.bluetooth.BluetoothProfile
import android.bluetooth.le.ScanCallback
import android.bluetooth.le.ScanFilter
import android.bluetooth.le.ScanResult
import android.bluetooth.le.ScanSettings
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.os.Build
import android.os.IBinder
import android.util.Log
import androidx.core.app.NotificationChannelCompat
import androidx.core.app.NotificationCompat
import androidx.core.app.NotificationManagerCompat
import androidx.core.content.ContextCompat
import com.luojigou.xiaodou.ble.XDBLEUtils
import com.luojigou.xiaodou.ble.XDScanBLEDevice
import com.luojigou.xiaodou.wifi.XDScanWifiInfo
import java.util.UUID
import kotlin.concurrent.thread


@SuppressLint("MissingPermission")
class XDScanBLEService : Service() {
    companion object {
        const val ACTION_START_SCAN = "com.luojigou.xiaodou.ble.ACTION_START_SCAN"
        const val ACTION_STOP_SCAN = "com.luojigou.xiaodou.ble.ACTION_STOP_SCAN"
        const val ACTION_RESTART_SCAN = "com.luojigou.xiaodou.ble.ACTION_RESTART_SCAN"

        const val ACTION_SCAN_RESULT = "com.luojigou.xiaodou.ble.ACTION_SCAN_RESULT"
        const val ACTION_SCAN_RESULT_DATA = "com.luojigou.xiaodou.ble.ACTION_SCAN_RESULT_DATA"

        const val ACTION_SELECT_BLE_DEVICE = "com.luojigou.xiaodou.ble.ACTION_SELECT_BLE_DEVICE"
        const val ACTION_SELECT_BLE_DEVICE_DATA = "com.luojigou.xiaodou.ble.ACTION_SELECT_BLE_DEVICE_DATA"
        const val ACTION_SELECT_WIFI_DEVICE = "com.luojigou.xiaodou.ble.ACTION_SELECT_WIFI_DEVICE"
        const val ACTION_SELECT_WIFI_DEVICE_DATA = "com.luojigou.xiaodou.ble.ACTION_SELECT_WIFI_DEVICE_DATA"

        const val ACTION_CONNECT_WIFI = "com.luojigou.xiaodou.ble.ACTION_CONNECT_WIFI"
        const val ACTION_CONNECT_WIFI_RESULT_ERROR = "com.luojigou.xiaodou.ble.ACTION_CONNECT_WIFI_RESULT_ERROR"
        const val ACTION_CONNECT_WIFI_RESULT_SUCCESS = "com.luojigou.xiaodou.ble.ACTION_CONNECT_WIFI_RESULT_SUCCESS"
        const val ACTION_CONNECT_WIFI_RESULT_FAILED = "com.luojigou.xiaodou.ble.ACTION_CONNECT_WIFI_RESULT_FAILED"

        private val wifiConnectServiceUUID = UUID.fromString("0000ae80-0000-1000-8000-00805f9b34fb")
        private val wifiConnectWriteUUID = UUID.fromString("0000ae81-0000-1000-8000-00805f9b34fb")
        private val wifiConnectNotifyUUID = UUID.fromString("0000ae82-0000-1000-8000-00805f9b34fb")

        private val bleChannelName = "BLE-CHANNEL"
    }

    private val bluetoothDevices: MutableMap<XDScanBLEDevice, BluetoothDevice> = HashMap()

    private lateinit var bluetoothManager: BluetoothManager

    private lateinit var bluetoothAdapter: BluetoothAdapter

    /// 选择要连接的蓝牙设备
    private var selectBluetoothDevice: XDScanBLEDevice? = null

    /// 选择要连接的wifi网络
    private var selectWifiInfo: XDScanWifiInfo? = null

    private var bluetoothReceiver: BroadcastReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent) {
            when (intent.action) {
                ACTION_RESTART_SCAN -> {
                    bluetoothDevices.clear()
                    startScan()
                }

                ACTION_CONNECT_WIFI -> {
                    val device = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
                        intent.getParcelableExtra("device", XDScanBLEDevice::class.java)
                    } else {
                        intent.getParcelableExtra<XDScanBLEDevice>("device")
                    }
                    val ssid = intent.getStringExtra("ssid")
                    val password = intent.getStringExtra("password")
                    connectWifi(device!!, ssid!!, password!!)
                }

                ACTION_SELECT_BLE_DEVICE -> {
                    val bleDevice = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
                        intent.getParcelableExtra(
                            ACTION_SELECT_BLE_DEVICE_DATA, XDScanBLEDevice::class.java
                        )
                    } else {
                        intent.getParcelableExtra<XDScanBLEDevice>(ACTION_SELECT_BLE_DEVICE)
                    }

                    if (bleDevice != null) {
                        selectBluetoothDevice = bleDevice
                        selectWifiInfo = null
                    }
                }

                ACTION_SELECT_WIFI_DEVICE -> {
                    val wifiInfo = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
                        intent.getParcelableExtra(
                            ACTION_SELECT_WIFI_DEVICE_DATA, XDScanWifiInfo::class.java
                        )
                    } else {
                        intent.getParcelableExtra<XDScanWifiInfo>(ACTION_SELECT_WIFI_DEVICE)
                    }

                    if (wifiInfo != null) {
                        selectWifiInfo = wifiInfo
                        if (selectBluetoothDevice != null) {
                            connectWifi(selectBluetoothDevice!!, wifiInfo.ssid, wifiInfo.password)
                        }
                    }
                }
            }
        }
    }

    @Volatile
    private var bleScanning = false

    private val bleScanSettings = ScanSettings.Builder()
        .setScanMode(ScanSettings.SCAN_MODE_LOW_LATENCY)
        .build()

    private val bleScanFilters = mutableListOf<ScanFilter>()

    private val bleScanCallback = object : ScanCallback() {
        override fun onScanResult(callbackType: Int, result: ScanResult) {
            super.onScanResult(callbackType, result)
            val oldSize = bluetoothDevices.size
            addDevice(result)
            val newSize = bluetoothDevices.size

            if (oldSize != newSize) {
                sendResult()
            }
        }

        override fun onBatchScanResults(results: MutableList<ScanResult>) {
            super.onBatchScanResults(results)
            val oldSize = bluetoothDevices.size
            results.forEach { addDevice(it) }
            val newSize = bluetoothDevices.size

            if (oldSize != newSize) {
                sendResult()
            }

        }

        override fun onScanFailed(errorCode: Int) {
            super.onScanFailed(errorCode)
            Log.d("XDScanBLEService", "onScanFailed: $errorCode")
        }

        private fun addDevice(result: ScanResult) {
            result.apply {
                if (device.name == null) return

                val device = XDScanBLEDevice(
                    name = device.name ?: "unknown",
                    bondState = device.bondState.toString(),
                    rssi = rssi.toString(),
                    address = device.address,
                    addressType = device.type.toString(),
                )

                if (!bluetoothDevices.contains(device)) {
                    bluetoothDevices[device] = result.device
                }
            }

        }
    }

    override fun onCreate() {
        super.onCreate()
        goForeground()
        initBluetooth()
    }

    private fun goForeground() {
        val manager = NotificationManagerCompat.from(this)
        val notificationChannel = NotificationChannelCompat.Builder(
            bleChannelName, NotificationManagerCompat.IMPORTANCE_LOW
        ).setName("蓝牙扫描通知").setDescription("用于蓝牙扫描硬件设备并配网处理").build()

        if (manager.areNotificationsEnabled()) {
            if (manager.getNotificationChannel(bleChannelName) == null) {
                manager.createNotificationChannel(notificationChannel)
            }

            val notification = NotificationCompat.Builder(this, bleChannelName)
                .setContentTitle("小逗AI设备配网服务")
                .setContentText("正在为小逗AI设备配网中")
                .setSmallIcon(android.R.drawable.stat_sys_data_bluetooth)
                .setChannelId(bleChannelName)
                .build()

            startForeground(1, notification)
        }
    }

    private fun initBluetooth() {
        bluetoothManager = getSystemService(Context.BLUETOOTH_SERVICE) as BluetoothManager
        bluetoothAdapter = bluetoothManager.adapter

        val intentFilter = IntentFilter()
        intentFilter.addAction(ACTION_RESTART_SCAN)
        intentFilter.addAction(ACTION_SELECT_BLE_DEVICE)
        intentFilter.addAction(ACTION_SELECT_WIFI_DEVICE)
        intentFilter.addAction(ACTION_CONNECT_WIFI)
        ContextCompat.registerReceiver(
            this, bluetoothReceiver, intentFilter, ContextCompat.RECEIVER_EXPORTED
        )

        startScan()
    }

    override fun onBind(intent: Intent?): IBinder? {
        return null
    }

    private fun startScan() {
        if (!bleScanning) {
            bleScanning = true
            bluetoothAdapter.bluetoothLeScanner.startScan(
                bleScanFilters, bleScanSettings, bleScanCallback
            )
        }
        delayClose()
    }

    private fun delayClose() {
        thread {
            sendBroadcast(Intent(ACTION_START_SCAN))
            Thread.sleep(5000)
            if (bleScanning) {
                stopScan()
            }
            Thread.sleep(3000)
            sendBroadcast(Intent(ACTION_STOP_SCAN))
        }
    }

    private fun stopScan() {
        if (bleScanning) {
            bleScanning = false
            bluetoothAdapter.bluetoothLeScanner.stopScan(bleScanCallback)
        }
    }

    private fun sendResult() {
        val intent = Intent(ACTION_SCAN_RESULT)
        intent.putParcelableArrayListExtra(
            ACTION_SCAN_RESULT_DATA, ArrayList(bluetoothDevices.keys)
        )
        sendBroadcast(intent)
    }

    private fun connectWifi(device: XDScanBLEDevice, ssid: String, password: String) {
        val bleDevice = bluetoothDevices[device] ?: return

        val gattCallback = object : BluetoothGattCallback() {
            override fun onConnectionStateChange(gatt: BluetoothGatt, status: Int, newState: Int) {
                super.onConnectionStateChange(gatt, status, newState)
                Log.d("XDScanBLEGattCallback", "onConnectionStateChange: $status,$newState")
                when (newState) {
                    BluetoothProfile.STATE_CONNECTED -> {
                        gatt.discoverServices()
                    }

                    BluetoothProfile.STATE_DISCONNECTED -> {
                        gatt.close()
                    }
                }
            }

            override fun onServicesDiscovered(gatt: BluetoothGatt, status: Int) {
                super.onServicesDiscovered(gatt, status)
                Log.d("XDScanBLEGattCallback", "onServicesDiscovered: $status")

                when (status) {
                    BluetoothGatt.GATT_SUCCESS -> {
                        val service = gatt.getService(wifiConnectServiceUUID)!!

                        val notifyCharacteristic = service.getCharacteristic(wifiConnectNotifyUUID)!!
                        val writeCharacteristic = service.getCharacteristic(wifiConnectWriteUUID)!!

                        gatt.setCharacteristicNotification(notifyCharacteristic, true)


                    }
                }
            }

            override fun onCharacteristicChanged(
                gatt: BluetoothGatt, characteristic: BluetoothGattCharacteristic, value: ByteArray
            ) {
                super.onCharacteristicChanged(gatt, characteristic, value)
                val result = XDBLEUtils.parseConnectResponse(value)
                Log.d("XDScanBLEService", "onCharacteristicChanged: $result")
            }
        }

        val connectGatt = bleDevice.connectGatt(
            this, false, gattCallback, BluetoothDevice.TRANSPORT_LE
        )
    }

    override fun onDestroy() {
        unregisterReceiver(bluetoothReceiver)
        stopScan()
        super.onDestroy()
        Log.d("XDScanBLEService", "onDestroy: service destroyed")
    }
}